# SPDX-FileCopyrightText: 2023-present Guillaume Collet <bilouweb@free.fr>
#
# SPDX-License-Identifier: CECILL-2.1
import pytest

from wasc import utils


class TestReadCheckers:
    def test_no_file(self):
        with pytest.raises(FileNotFoundError):
            utils.read_checkers("foo.txt")
    def test_read_crit_example(self):
        expected_checkers = {
            "HeadNbChecker", "HeadLvlChecker",
            "AccessChecker", "AccessLinkChecker", "AccessRateChecker",
            "LegalChecker", "LangChecker", "DoctypeChecker"
        }
        read_checkers = utils.read_checkers("tests/data/checkers_example.csv")
        assert isinstance(read_checkers, list)
        assert set(read_checkers) == expected_checkers

class TestReadWebsites:
    def test_no_file(self):
        with pytest.raises(FileNotFoundError):
            utils.read_websites("foo.txt")
    def test_read_url_example(self):
        expected_url = [
            ("Design Gouv", "https://design.numerique.gouv.fr/"),
            ("Example", "http://example.com")
        ]
        url_example = utils.read_websites("tests/data/url_example.csv")
        assert isinstance(url_example, list)
        for website in url_example:
            assert website in expected_url

EXAMPLE_TEST_URL = "https://www.example.com/fr/test"
EXAMPLE_ROOT = "https://www.example.com/fr"
EXAMPLE_ROOT_SLASH = "https://www.example.com/fr/"

class TestCheckAndCorrectUrl:
    def test_empty(self):
        assert not utils.check_and_correct_url("", "")

    def test_correct(self):
        assert utils.check_and_correct_url(EXAMPLE_TEST_URL, "https://www.example.com") == EXAMPLE_TEST_URL
        assert utils.check_and_correct_url("/fr/test/", EXAMPLE_ROOT_SLASH) == EXAMPLE_TEST_URL
        assert utils.check_and_correct_url("fr/test/", EXAMPLE_ROOT_SLASH) == EXAMPLE_TEST_URL
        assert utils.check_and_correct_url("/fr/test/", EXAMPLE_ROOT) == EXAMPLE_TEST_URL



# SPDX-FileCopyrightText: 2023-present Guillaume Collet <bilouweb@free.fr>
#
# SPDX-License-Identifier: CECILL-2.1
import wasc.checker_factory as fact


class TmpChecker:
    pass

class TestCheckerFactory:
    def test_checker_factory_init(self):
        tmp_factory = fact.CheckerFactory()
        assert isinstance(tmp_factory, fact.CheckerFactory)
    def test_checker_factory_register(self):
        tmp_factory = fact.CheckerFactory()
        tmp_factory.register("tmp_name", TmpChecker)
        assert tmp_factory.is_registered("tmp_name")
    def test_checker_factory_create(self):
        tmp_factory = fact.CheckerFactory()
        tmp_factory.register("tmp_name", TmpChecker)
        tmp_obj = tmp_factory.create("tmp_name")
        assert isinstance(tmp_obj, TmpChecker)
    def test_checker_factory_available(self):
        tmp_factory = fact.CheckerFactory()
        tmp_factory.register("tmp_name", TmpChecker)
        check_list = tmp_factory.available()
        assert check_list == ["tmp_name"]



# SPDX-FileCopyrightText: 2023-present Guillaume Collet <bilouweb@free.fr>
#
# SPDX-License-Identifier: CECILL-2.1

import re

import bs4
from trafilatura import extract, fetch_url

from wasc.abstract_checker import AbstractChecker
from wasc.utils import FAIL, OK, PRESENT, check_and_correct_url

mentions = "non|partiellement|totalement"
ACCESS_PATTERN = re.compile("Accessibilité[ \xa0]:[ \xa0](" + mentions + ")[ \xa0]conforme", re.IGNORECASE)

class HeadNbChecker(AbstractChecker) :
    """HeadNbChecker
    A class to test the number of <head> tags in a page.
    """
    def __init__(self) :
        super().__init__("HeadNbChecker", "Nombre de <head>")

    def execute(self, web_page : bs4.BeautifulSoup, root_url : str):  # noqa: ARG002
        return len(web_page.find_all(name="head"))

class HeadLvlChecker(AbstractChecker) :
    """HeadLvlChecker
    Get the depth of <head> tags in a web page.
    """
    def __init__(self) :
        super().__init__("HeadLvlChecker", "Profondeurs des <head>")

    def execute(self, web_page : bs4.BeautifulSoup, root_url : str):  # noqa: ARG002
        head_tag = web_page.find_all("head")
        return [len(list(tag.parents)) - 1 for tag in head_tag] if head_tag else []

class AccessChecker(AbstractChecker) :
    """AccessChecker
    Check the presence of "Accessibilité" RGAA4 mention on the web page.
        1 - Search in <footer>
        2 - Search in <div id="footer">
        3 - Search in the whole page
    """
    def __init__(self) :
        super().__init__("AccessChecker", "Mention accessibilité")

    def execute(self, web_page : bs4.BeautifulSoup, root_url : str):  # noqa: ARG002
        mention = ""
        footer = web_page.footer
        if footer:
            mention = footer.find_all(string = ACCESS_PATTERN)
        else:
            footer = web_page.find(id="footer")
            if footer:
                mention = footer.find_all(string = ACCESS_PATTERN)
            else:
                mention = web_page.find_all(string = ACCESS_PATTERN)
        return mention[0].split(":")[1].strip() if mention else FAIL

class AccessLinkChecker(AbstractChecker) :
    """AccessLinkChecker
    Check if a link ot an accessibility statement exists
    Search for a link as defined in search_link
        1 - in <footer>
        2 - in <div id="footer">
        3 - in the whole page
    """
    def __init__(self) :
        super().__init__("AccessLinkChecker", "Lien accessibilité")

    def search_link(self, web_page, root_url):
        # 1 - Try to find link in Mention Accessibilité
        access_tag = web_page.find("a", string=ACCESS_PATTERN)
        if access_tag :
            try:
                return check_and_correct_url(access_tag.attrs["href"], root_url)
            except Exception:
                return FAIL
        # 2 - Try to find text "accessibilité" in a link
        access_tag = web_page.find("a", string=re.compile("accessibilit", re.IGNORECASE))
        if access_tag :
            try:
                return check_and_correct_url(access_tag.attrs["href"], root_url)
            except Exception:
                return FAIL
        # 3 - Try to find a link that contains accessibilit in href
        link_tag = web_page.find("a", href=re.compile("accessibilit", re.IGNORECASE))
        if link_tag:
            try:
                return check_and_correct_url(link_tag.attrs["href"], root_url)
            except Exception:
                return FAIL
        return FAIL

    def execute(self, web_page : bs4.BeautifulSoup, root_url : str):
        footer = web_page.footer
        if footer:
            result = self.search_link(footer, root_url)
            if result != FAIL:
                return result
        footer = web_page.find(id="footer")
        if footer:
            result = self.search_link(footer, root_url)
            if result != FAIL:
                return result
        return self.search_link(web_page, root_url)

class AccessRateChecker(AbstractChecker) :
    """AccessRateChecker
    Returns the compliance rate (%) on the accessibility statement if found
    """
    def __init__(self) :
        super().__init__("AccessRateChecker", "Taux d'accessibilité")

    def execute(self, web_page : bs4.BeautifulSoup, root_url : str):
        link_url = AccessLinkChecker().execute(web_page, root_url)
        if link_url == FAIL:
            return FAIL
        try:
            response = fetch_url(link_url)
            if response:
                result = extract(response, output_format="xml", include_links=True, no_fallback=True)
                link_page = bs4.BeautifulSoup(result, "html.parser")
                motif = re.compile(r"%", re.IGNORECASE)
                percent_tags = link_page.find_all(string = motif)
                for tag in percent_tags:
                    if "conformité" in tag:
                        m = re.search(r"\s(100|(\d{1,2}([\.\,]\d+)*)) *%", str(tag))
                        if m:
                            return str(m[1]) + "%"
        except Exception:
            return FAIL
        return FAIL

class LegalChecker(AbstractChecker) :
    """LegalChecker
    Test the presence of "Mentions légales" link on the web page.
    Try to find :
        * text "Mentions légales" (case insensitive) and check if a link exists
        * try the url root_url + "/mentions-legales" and check if a link exists
    Return the link to "Mentions légales" page if it exists, else fail
    """
    def __init__(self) :
        super().__init__("LegalChecker", "Mentions légales")

    def execute(self, web_page : bs4.BeautifulSoup, root_url : str):
        motif = re.compile("Mentions* l[eé]gales*", re.IGNORECASE)
        legal_tags = web_page.find_all(string = motif)
        for tag in legal_tags:
            legal_tag = tag
            while legal_tag and legal_tag.name != "a" and legal_tag.name != "html":
                legal_tag = legal_tag.parent
            try :
                legal_link = check_and_correct_url(legal_tag.attrs["href"], root_url)
                return check_and_correct_url(legal_tag.attrs["href"], root_url)
            except KeyError :
                pass
        legal_link = check_and_correct_url("mentions-legales", root_url)
        try:
            response = fetch_url(legal_link, decode=False)
            if response and response.status == OK :
                return legal_link
        except Exception:
            return FAIL
        return FAIL

class LangChecker(AbstractChecker) :
    """LangChecker
    Check the presence of attribute lang in the html tag of the website
    """
    def __init__(self) :
        super().__init__("LangChecker", "Lang")

    def execute(self, web_page : bs4.BeautifulSoup, root_url : str):  # noqa: ARG002
        try :
            if isinstance(web_page.html, bs4.Tag):
                return web_page.html.attrs["lang"]
        except KeyError :
            return FAIL

class DoctypeChecker(AbstractChecker) :
    """DoctypeChecker
    Check the presence of <!DOCTYPE html> at the beginning of HTML document (before <html>)
    """
    def __init__(self) :
        super().__init__("DoctypeChecker", "Doctype")

    def execute(self, web_page : bs4.BeautifulSoup, root_url : str):  # noqa: ARG002
        current_pos = 0
        doctype_pos = 1
        html_pos = 0
        doctype_found = False
        for item in web_page.contents:
            if isinstance(item, bs4.Doctype):
                doctype_pos = current_pos
                doctype_found = True
                if item != "html":
                    return FAIL
            elif isinstance(item, bs4.Tag) and item.name == "html":
                html_pos = current_pos
            current_pos += 1
        if doctype_found and doctype_pos < html_pos:
            return "html"
        return FAIL

class HeaderChecker(AbstractChecker) :
    """HeaderChecker
    Check the presence of a unique <header> tag
    """
    def __init__(self) :
        super().__init__("HeaderChecker", "Header")

    def execute(self, web_page : bs4.BeautifulSoup, root_url : str):  # noqa: ARG002
        return PRESENT if len(web_page.find_all(name="header")) == 1 else FAIL

class FooterChecker(AbstractChecker) :
    """FooterChecker
    Check the presence of a unique <footer> tag
    """
    def __init__(self) :
        super().__init__("FooterChecker", "Footer")

    def execute(self, web_page : bs4.BeautifulSoup, root_url : str):  # noqa: ARG002
        return PRESENT if len(web_page.find_all(name="footer")) == 1 else FAIL

class ContactLinkChecker(AbstractChecker) :
    """ContactLinkChecker
    Check the presence of contact link in the page
    """
    def __init__(self) :
        super().__init__("ContactLinkChecker", "Lien Contact")

    def execute(self, web_page : bs4.BeautifulSoup, root_url : str):
        link_tags = web_page.find_all(href=re.compile("(contact|ecrire)"))
        for tag in link_tags:
            try :
                return check_and_correct_url(tag.attrs["href"], root_url)
            except KeyError :
                pass
        return FAIL



# SPDX-FileCopyrightText: 2023-present Guillaume Collet <bilouweb@free.fr>
#
# SPDX-License-Identifier: CECILL-2.1
from bs4 import BeautifulSoup

import wasc.checkers as dft
from wasc.utils import FAIL, PRESENT

BS_PARSER = "html.parser"

DEFAULT_HTML_HEAD = "<!DOCTYPE html><html><head></head><body><div>"
DEFAULT_HTML_TAIL = "</div></body></html>"
DEFAULT_HTML_ROOT = "https://www.example.com"
DESIGN_NUM = "https://design.numerique.gouv.fr"
HTML_BODY_ONLY = "<!DOCTYPE html><html><body></body></html>"

class TestDoctypeChecker:
    def test_doctype_checker_init(self):
        doctype_checker = dft.DoctypeChecker()
        assert doctype_checker.name == "DoctypeChecker"
        assert doctype_checker.description == "Doctype"

    def test_doctype_checker_valid(self):
        test_html = "<!DOCTYPE html><html></html>"
        doctype_checker = dft.DoctypeChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert doctype_checker.execute(basic_webpage, "") == "html"

    def test_doctype_checker_bad_doctype(self):
        test_html = "<!DOCTYPE notvalid><html></html>"
        doctype_checker = dft.DoctypeChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert doctype_checker.execute(basic_webpage, "") == FAIL

    def test_doctype_checker_no_doctype(self):
        test_html = "<html></html>"
        doctype_checker = dft.DoctypeChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert doctype_checker.execute(basic_webpage, "") == FAIL

class TestLangChecker:
    def test_lang_checker_init(self):
        lang_checker = dft.LangChecker()
        assert lang_checker.name == "LangChecker"
        assert lang_checker.description == "Lang"

    def test_lang_checker_valid(self):
        test_html = '<!DOCTYPE html><html lang="fr"></html>'
        lang_checker = dft.LangChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert lang_checker.execute(basic_webpage, "") == "fr"

    def test_lang_checker_empty(self):
        test_html = "<!DOCTYPE html><html></html>"
        lang_checker = dft.LangChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert lang_checker.execute(basic_webpage, "") == FAIL

class TestAccessChecker:
    def test_access_checker_init(self):
        access_checker = dft.AccessChecker()
        assert access_checker.name == "AccessChecker"
        assert access_checker.description == "Mention accessibilité"

    def test_access_checker_fail1(self):
        test_html = DEFAULT_HTML_HEAD + DEFAULT_HTML_TAIL
        access_checker = dft.AccessChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert access_checker.execute(basic_webpage, "") == FAIL

    def test_access_checker_fail2(self):
        test_html = DEFAULT_HTML_HEAD + "Accessibilité : conforme partiellement" + DEFAULT_HTML_TAIL
        access_checker = dft.AccessChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert access_checker.execute(basic_webpage, "") == FAIL

    def test_access_checker_valid_non(self):
        test_html = DEFAULT_HTML_HEAD + "Accessibilité : non conforme" + DEFAULT_HTML_TAIL
        access_checker = dft.AccessChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert access_checker.execute(basic_webpage, "") == "non conforme"

    def test_access_checker_valid_partiel(self):
        test_html = DEFAULT_HTML_HEAD + "Accessibilité : partiellement conforme" + DEFAULT_HTML_TAIL
        access_checker = dft.AccessChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert access_checker.execute(basic_webpage, "") == "partiellement conforme"

    def test_access_checker_valid_total(self):
        test_html = DEFAULT_HTML_HEAD + "Accessibilité : totalement conforme" + DEFAULT_HTML_TAIL
        access_checker = dft.AccessChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert access_checker.execute(basic_webpage, "") == "totalement conforme"

class TestAccessLinkChecker:
    def test_access_link_checker_init(self):
        access_link_checker = dft.AccessLinkChecker()
        assert access_link_checker.name == "AccessLinkChecker"
        assert access_link_checker.description == "Lien accessibilité"

    def test_access_link_checker_valid_mention(self):
        test_link = '<a href="/accessibilite/">Accessibilité : non conforme</a>'
        test_html = DEFAULT_HTML_HEAD + test_link + DEFAULT_HTML_TAIL
        access_link_checker = dft.AccessLinkChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = DEFAULT_HTML_ROOT + "/accessibilite"
        assert access_link_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == answer

    def test_access_link_checker_fail_mention(self):
        test_html = DEFAULT_HTML_HEAD + "Accessibilité : totalement conforme" + DEFAULT_HTML_TAIL
        access_link_checker = dft.AccessLinkChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert access_link_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == FAIL

    def test_access_link_checker_valid_other(self):
        test_html = DEFAULT_HTML_HEAD + '<a href="/accessibilite/">Accessibilité</a>' + DEFAULT_HTML_TAIL
        access_link_checker = dft.AccessLinkChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert access_link_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == DEFAULT_HTML_ROOT + "/accessibilite"

    def test_access_link_checker_fail_other(self):
        test_html = DEFAULT_HTML_HEAD + '<a href="/misc/accessibilite/">Accessibilité</a>' + DEFAULT_HTML_TAIL
        access_link_checker = dft.AccessLinkChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = DEFAULT_HTML_ROOT + "/misc/accessibilite"
        assert access_link_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == answer

    def test_access_link_checker_fail_href(self):
        test_html = DEFAULT_HTML_HEAD + "<a>Accessibilité</a>" + DEFAULT_HTML_TAIL
        access_link_checker = dft.AccessLinkChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert access_link_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == FAIL

    def test_access_link_checker_decla(self):
        test_link = '<a href="/misc/accessibilite/">Déclaration d\'accessibilité</a>'
        test_html = DEFAULT_HTML_HEAD + test_link + DEFAULT_HTML_TAIL
        access_link_checker = dft.AccessLinkChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = DEFAULT_HTML_ROOT + "/misc/accessibilite"
        assert access_link_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == answer

class TestAccessRateChecker:
    def test_access_rate_checker_init(self):
        access_rate_checker = dft.AccessRateChecker()
        assert access_rate_checker.name == "AccessRateChecker"
        assert access_rate_checker.description == "Taux d'accessibilité"

    def test_access_rate_checker_valid(self):
        """
        Be Careful, this test use a real url that may change over time
        """
        test_link = '<a href="/misc/accessibilite/">Accessibilité : totalement conforme</a>'
        test_html = DEFAULT_HTML_HEAD + test_link + DEFAULT_HTML_TAIL
        access_rate_checker = dft.AccessRateChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert access_rate_checker.execute(basic_webpage, DESIGN_NUM) == "100%"

    def test_access_rate_checker_fail_link(self):
        test_html = DEFAULT_HTML_HEAD + "Accessibilité : non conforme" + DEFAULT_HTML_TAIL
        access_rate_checker = dft.AccessRateChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert access_rate_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == FAIL

    def test_access_rate_checker_fail_link2(self):
        test_link = '<a href="/misc/accessibilite/">Accessibilité : totalement conforme</a>'
        test_html = DEFAULT_HTML_HEAD + test_link + DEFAULT_HTML_TAIL
        access_rate_checker = dft.AccessRateChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert access_rate_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == FAIL

class TestLegalChecker:
    def test_mention_legales_checker_init(self):
        mention_legales_checker = dft.LegalChecker()
        assert mention_legales_checker.name == "LegalChecker"
        assert mention_legales_checker.description == "Mentions légales"

    def test_mention_legales_valid_mention1(self):
        test_html = DEFAULT_HTML_HEAD + '<a href="/misc/mentions-legales/">Mentions légales</a>' + DEFAULT_HTML_TAIL
        mention_legales_checker = dft.LegalChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = DEFAULT_HTML_ROOT + "/misc/mentions-legales"
        assert mention_legales_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == answer

    def test_mention_legales_valid_mention2(self):
        test_html = DEFAULT_HTML_HEAD + '<a href="/misc/mentions-legales/">mention legale</a>' + DEFAULT_HTML_TAIL
        mention_legales_checker = dft.LegalChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = DEFAULT_HTML_ROOT + "/misc/mentions-legales"
        assert mention_legales_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == answer

    def test_mention_legales_fail_mention(self):
        test_html = DEFAULT_HTML_HEAD + "Mentions légales" + DEFAULT_HTML_TAIL
        mention_legales_checker = dft.LegalChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert mention_legales_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == FAIL

    def test_mention_legales_valid_dftlink(self):
        test_html = DEFAULT_HTML_HEAD + "foo" + DEFAULT_HTML_TAIL
        mention_legales_checker = dft.LegalChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = "https://www.gouvernement.fr/mentions-legales"
        assert mention_legales_checker.execute(basic_webpage, "https://www.gouvernement.fr/") == answer

class TestHeadNbChecker:
    def test_head_nb_checker_init(self):
        head_nb_checker = dft.HeadNbChecker()
        assert head_nb_checker.name == "HeadNbChecker"
        assert head_nb_checker.description == "Nombre de <head>"
    def test_head_nb_valid_01(self):
        test_html = DEFAULT_HTML_HEAD + DEFAULT_HTML_TAIL
        head_nb_checker = dft.HeadNbChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = 1
        assert head_nb_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == answer
    def test_head_nb_valid_02(self):
        test_html = DEFAULT_HTML_HEAD + "<head></head>" + DEFAULT_HTML_TAIL
        head_nb_checker = dft.HeadNbChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = 2
        assert head_nb_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == answer
    def test_head_nb_fail(self):
        test_html = HTML_BODY_ONLY
        head_nb_checker = dft.HeadNbChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = 0
        assert head_nb_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == answer

class TestHeadLvlChecker:
    def test_head_lvl_checker_init(self):
        head_lvl_checker = dft.HeadLvlChecker()
        assert head_lvl_checker.name == "HeadLvlChecker"
        assert head_lvl_checker.description == "Profondeurs des <head>"
    def test_head_lvl_valid_01(self):
        test_html = DEFAULT_HTML_HEAD + DEFAULT_HTML_TAIL
        head_lvl_checker = dft.HeadLvlChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = [1]
        assert head_lvl_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == answer
    def test_head_lvl_valid_02(self):
        test_html = DEFAULT_HTML_HEAD + "<head></head>" + DEFAULT_HTML_TAIL
        head_lvl_checker = dft.HeadLvlChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = [1,3]
        assert head_lvl_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == answer
    def test_head_lvl_fail(self):
        test_html = HTML_BODY_ONLY
        head_lvl_checker = dft.HeadLvlChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        answer = []
        assert head_lvl_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == answer

class TestHeaderChecker:
    def test_header_checker_init(self):
        header_checker = dft.HeaderChecker()
        assert header_checker.name == "HeaderChecker"
        assert header_checker.description == "Header"

    def test_header_checker_fail(self):
        test_html = HTML_BODY_ONLY
        header_checker = dft.HeaderChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert header_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == FAIL

    def test_header_checker_present(self):
        test_html = "<!DOCTYPE html><html><body><header></header></body></html>"
        header_checker = dft.HeaderChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert header_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == PRESENT

class TestFooterChecker:
    def test_footer_checker_init(self):
        footer_checker = dft.FooterChecker()
        assert footer_checker.name == "FooterChecker"
        assert footer_checker.description == "Footer"

    def test_footer_checker_fail(self):
        test_html = HTML_BODY_ONLY
        footer_checker = dft.FooterChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert footer_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == FAIL

    def test_footer_checker_present(self):
        test_html = "<!DOCTYPE html><html><body><footer></footer></body></html>"
        footer_checker = dft.FooterChecker()
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        assert footer_checker.execute(basic_webpage, DEFAULT_HTML_ROOT) == PRESENT

class TestContactLinkChecker:
    def test_contact_link_checker_init(self):
        contact_link_checker = dft.ContactLinkChecker()
        assert contact_link_checker.name == "ContactLinkChecker"
        assert contact_link_checker.description == "Lien Contact"

    def test_contact_link_checker_valid(self):
        test_link = '<a href="/contact">Nous contacter</a>'
        test_html = DEFAULT_HTML_HEAD + test_link + DEFAULT_HTML_TAIL
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        contact_link_checker = dft.ContactLinkChecker()
        answer = "https://design.numerique.gouv.fr/contact"
        assert contact_link_checker.execute(basic_webpage, DESIGN_NUM) == answer

    def test_contact_link_checker_no_href(self):
        test_link = "<a>Nous contacter</a>"
        test_html = DEFAULT_HTML_HEAD + test_link + DEFAULT_HTML_TAIL
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        contact_link_checker = dft.ContactLinkChecker()
        assert contact_link_checker.execute(basic_webpage, DESIGN_NUM) == FAIL

    def test_contact_link_checker_bad_link(self):
        test_link = '<a href="/foo">Nous contacter</a>'
        test_html = DEFAULT_HTML_HEAD + test_link + DEFAULT_HTML_TAIL
        basic_webpage = BeautifulSoup(test_html, BS_PARSER)
        contact_link_checker = dft.ContactLinkChecker()
        assert contact_link_checker.execute(basic_webpage, DESIGN_NUM) == FAIL



# SPDX-FileCopyrightText: 2023-present Guillaume Collet <bilouweb@free.fr>
#
# SPDX-License-Identifier: CECILL-2.1
"""
Module abstract_checker

Provides an AbstractChecker class which defines how to create checkers.
A checker is meant to be the smallest possible test on website content.
"""
import abc

from bs4 import BeautifulSoup


class AbstractChecker(abc.ABC) :
    """AbstractChecker
    Abstract Checker class declares the abstract method execute() that
    needs to be implemented in child classes
    """
    def __init__(self, name :str, description : str) :
        """
        Sets the name and description of checkers
        """
        self.__name = name
        self.__description = description

    @property
    def name(self) :
        """
        Returns the name of the checker
        """
        return self.__name

    @property
    def description(self) :
        """
        Returns the description of the checker
        """
        return self.__description

    @abc.abstractmethod
    def execute(self, bs_object: BeautifulSoup, url: str):
        """
        Abstract method that needs to be implemented in subclasses.
        Execute the tests of the checker
        """



# SPDX-FileCopyrightText: 2023-present Guillaume Collet <bilouweb@free.fr>
#
# SPDX-License-Identifier: CECILL-2.1
"""
This module provides some reading functions
"""
from urllib.parse import urljoin, urlparse

import pandas as pd

HEADER = {
    "user-agent" : "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 \
        (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36" ,
    "referer" : "https://www.google.com/"
    }
OK = 200
FAIL = "échec"
PRESENT = "présent"

def read_checkers(filename) :
    """
    Reads the list of checkers.
    """
    with open(filename, encoding = "utf-8") as config_file :
        return [line.strip() for line in config_file]

def read_websites(filename) :
    """
    Reads the websites list, each website is a couple (label, URL)
    """
    df = pd.read_csv(filename, sep=";", comment="#", header = None, names=["org", "url"], skipinitialspace=True)
    return list(zip(df.org, df.url))

def check_and_correct_url(target_url : str, root_url : str) -> str :
    """
    This method check if the target_url is truncated and, if so,
    recompose from the root_url
    """
    root = urlparse(root_url)
    base_url = ""
    if root.scheme and root.hostname:
        base_url = root.scheme + "://" + root.hostname
    return str(urljoin(base_url, target_url.strip(" /")))

def find_link(access_tag, root_url):
    """
    Not used anymore but keep it, it may be useful
    get up until on a link tag -> then check href content
    """
    while access_tag and access_tag.name != "a" and access_tag.name != "html":
        access_tag = access_tag.parent
    try :
        return check_and_correct_url(access_tag.attrs["href"], root_url)
    except KeyError :
        pass



# SPDX-FileCopyrightText: 2023-present Guillaume Collet <bilouweb@free.fr>
#
# SPDX-License-Identifier: CECILL-2.1
import sys

if __name__ == "__main__":
    from wasc.cli import wasc

    sys.exit(wasc())



# SPDX-FileCopyrightText: 2023-present Guillaume Collet <bilouweb@free.fr>
#
# SPDX-License-Identifier: CECILL-2.1
import json
import sys

import bs4
import click
import pandas as pd
from tqdm import tqdm
from trafilatura.downloads import add_to_compressed_dict, buffered_downloads, load_download_buffer

from wasc.__about__ import __version__
from wasc.checker_factory import checker_factory
from wasc.utils import FAIL, OK, read_checkers, read_websites

CONTEXT_SETTINGS = {"help_option_names": ["-h", "--help"]}
DEFAULT_CHECKERS = [
    "AccessChecker", "AccessLinkChecker", "AccessRateChecker",
    "DoctypeChecker", "LangChecker", "LegalChecker",
    "ContactLinkChecker"
]

@click.command(context_settings=CONTEXT_SETTINGS)
@click.argument("websites", type=click.Path(exists=True))
@click.option("-c", "--checkers", type=click.Path(exists=True),
              help="Checkers list to use")
@click.option("-f", "--output_format", default = "json",
              type=click.Choice(["json", "csv"], case_sensitive=False),
              help="Output format [default=json]")
@click.option("-l", "--list_checkers", is_flag=True, default=False,
              help="List known checkers [default=False]")
@click.option("-o", "--output", default=sys.stdout,
              type=click.File("w"),
              help="Output file [default=stdout]")
@click.version_option(version=__version__, prog_name="wasc")
def wasc(websites, checkers, output_format, list_checkers, output):
    """
    Websites Accessibility Criteria Checker,
    helps to evaluate accessibility criteria on a list of websites

    WEBSITES is a CSV file containing a list of websites as couples
    "label";"URL"
    """
    # If list_checkers then list checkers and stops
    if list_checkers:
        for checker in checker_factory.available():
            click.echo(checker)
        return
    # Reads and creates checker list
    checker_names = DEFAULT_CHECKERS
    if checkers:
        click.echo(f"Read checkers from {checkers}")
        checker_names = read_checkers(checkers)
    else :
        click.echo("Use default checkers")
    checkers_list = [checker_factory.create(checker_name) for checker_name in checker_names]

    # Sets column names for DataFrame using checkers descriptions
    column_names = ["Organisation", "URL", "Erreur"] + [checker.description for checker in checkers_list]

    # Reads the list of web sites
    websites = read_websites(websites)
    websites_dict = dict(zip([ws[1].strip("/") for ws in websites], [ws[0] for ws in websites]))
    url_list = [ws[1] for ws in websites]

    # Launch analysis
    click.echo(f"Analysis of {len(websites)} websites...")
    threads = 8
    dl_dict = add_to_compressed_dict(url_list)
    mybuffer, dl_dict = load_download_buffer(dl_dict)
    results = []
    with tqdm(total=len(url_list)) as pbar:
        for url, response in buffered_downloads(mybuffer, threads, decode=False):
            label = websites_dict[url.strip("/")]
            bs_obj = None
            error = ""
            if response:
                if response.status == OK :
                    bs_obj = bs4.BeautifulSoup(response.data, "html.parser")
                else:
                    error = "HTML Error Status " + str(response.status)
            else:
                error = "Problème lors du téléchargement"
            starter = [label, url, error]
            analysis = [checker.execute(bs_obj, url) if bs_obj else FAIL for checker in checkers_list]
            results.append(starter + analysis)
            pbar.update(1)

    # Creates the DataFrame from results
    df = pd.DataFrame(results, columns=column_names)
    df.set_index(["Organisation"], inplace=True)

    # Output results given -o output and -f output_format
    if output == sys.stdout:
        click.echo("Results:")
    else :
        click.echo("Save results in " + output.name)
    if output_format == "json":
        json_load = json.loads(df.to_json(force_ascii=False, orient="index"))
        click.echo(json.dumps(json_load, sort_keys=True, indent=4, ensure_ascii=False), file=output)
    elif output_format == "csv":
        click.echo(df.to_csv(sep=";"), file=output)
    click.echo("Completed")



# SPDX-FileCopyrightText: 2023-present Guillaume Collet <bilouweb@free.fr>
#
# SPDX-License-Identifier: CECILL-2.1
"""
Module checker_factory

Provides a Factory design pattern for instantiation of AbstractCheckers.

For further usage of the factory, import checker_factory object from this module.
"""
import wasc.checkers as chk


class CheckerFactory :
    """CheckerFactory
    Factory design pattern to record and to create Checkers objects
    """
    def __init__(self) :
        """
        Sets the empty dictionary checker_dict.
        """
        self.__checker_dict = {}

    def register(self, checker_name : str, checker_class) :
        """
        Registers a new checker by specifying its name and
        the associated checker subclass of AbstractChecker
        """
        self.__checker_dict[checker_name] = checker_class

    def create(self, checker_name : str):
        """
        Returns a Checker object given its name (must be registered name)
        """
        return self.__checker_dict[checker_name]()

    def available(self):
        """
        Returns the list of checkers names without order
        """
        return list(self.__checker_dict.keys())

    def is_registered(self, checker_name : str):
        """
        Returns True if checker_name exists in the dictionary of checkers
        """
        return checker_name in self.__checker_dict

checker_factory = CheckerFactory()
checker_factory.register("AccessChecker", chk.AccessChecker)
checker_factory.register("AccessLinkChecker", chk.AccessLinkChecker)
checker_factory.register("AccessRateChecker", chk.AccessRateChecker)
checker_factory.register("ContactLinkChecker", chk.ContactLinkChecker)
checker_factory.register("DoctypeChecker", chk.DoctypeChecker)
checker_factory.register("FooterChecker", chk.FooterChecker)
checker_factory.register("HeaderChecker", chk.HeaderChecker)
checker_factory.register("HeadLvlChecker", chk.HeadLvlChecker)
checker_factory.register("HeadNbChecker", chk.HeadNbChecker)
checker_factory.register("LangChecker", chk.LangChecker)
checker_factory.register("LegalChecker", chk.LegalChecker)
